﻿using Microsoft.TeamFoundation.Client; // Microsoft.TeamFoundation.Client (Add Reference > Extensions)
using Microsoft.TeamFoundation.ProcessConfiguration.Client; // Microsoft.TeamFoundation.ProjectManagement (Add Reference > Extensions)
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.WorkItemTracking.Client; // Microsoft.TeamFoundation.WorkItemTracking.Client (Add Reference > Extensions)
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration; // Reference System.Configuration
using System.Net;
using System.Text;
using System.Xml;

namespace Accentient
{
  class TFSHelper
  {
    private static TfsTeamProjectCollection TfsConnect()
    {
      try
      {
        // Pull settings from app.config

        string AccountURL = ConfigurationManager.AppSettings["AccountUrl"];
        string MicrosoftAccountAlias = ConfigurationManager.AppSettings["MicrosoftAccountAlias"];
        string MicrosoftAccountPwd = ConfigurationManager.AppSettings["MicrosoftAccountPassword"];

        // Authenticate
        NetworkCredential netCred = new NetworkCredential(MicrosoftAccountAlias, MicrosoftAccountPwd);
        BasicAuthCredential basicCred = new BasicAuthCredential(netCred);
        TfsClientCredentials tfsCred = new TfsClientCredentials(basicCred);
        tfsCred.AllowInteractive = false;
        TfsTeamProjectCollection tpc = new TfsTeamProjectCollection(new Uri(AccountURL), tfsCred);
        tpc.Authenticate();
        return tpc;
      }
      catch (Exception ex)
      {
        Console.WriteLine("Error: " + ex.Message);
        throw;
      }
    }

    #region Areas
    public static void RemoveAreas()
    {
      try
      {
        TfsTeamProjectCollection tpc = TfsConnect();
        string TeamProject = ConfigurationManager.AppSettings["TeamProject"];
        ICommonStructureService css = tpc.GetService<ICommonStructureService>();
        ProjectInfo project = css.GetProjectFromName(TeamProject);
        NodeInfo[] hierarchy = css.ListStructures(project.Uri);
        XmlElement tree = css.GetNodesXml(new string[] { hierarchy[0].Uri }, true);
        string parentUri = tree.FirstChild.Attributes["NodeID"].Value;

        // Enumerate nodes

        if (tree.HasChildNodes)
        {
          XmlNode childrenNode = tree.FirstChild;
          if (childrenNode.HasChildNodes && childrenNode.FirstChild.HasChildNodes)
          {
            string[] nodes = new string[childrenNode.ChildNodes[0].ChildNodes.Count];
            for (int i = 0; i < childrenNode.ChildNodes[0].ChildNodes.Count; i++)
            {
              XmlNode node = childrenNode.ChildNodes[0].ChildNodes[i];
              nodes[i] = node.Attributes["NodeID"].Value;
            }
            css.DeleteBranches(nodes, parentUri);
          }
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine("Error: " + ex.Message);
        throw;
      }
    }

    private static void AddArea(TfsTeamProjectCollection tpc, string teamProject, string nodeValue, string parentValue = "")
    {
      try
      {
        // Get service and hierarchy

        ICommonStructureService css = tpc.GetService<ICommonStructureService>();
        ProjectInfo project = css.GetProjectFromName(teamProject);
        NodeInfo[] hierarchy = css.ListStructures(project.Uri);
        XmlElement tree;
        tree = css.GetNodesXml(new string[] { hierarchy[0].Uri }, true);

        string parentUri = "";
        if (parentValue == "")
        {
          parentUri = tree.FirstChild.Attributes["NodeID"].Value;
        }
        else
        {
          parentUri = tree.SelectSingleNode("//Children/Node[@Name='" + parentValue + "']").Attributes["NodeID"].Value;
        }
        css.CreateNode(nodeValue, parentUri);
      }
      catch
      {
      }
    }

    public static void CreateAreas()
    {
      TfsTeamProjectCollection tpc = TfsConnect();

      // Load Areas

      XmlDocument xmlInput = new XmlDocument();
      xmlInput.Load(ConfigurationManager.AppSettings["InputFile"]);
      // XmlNodeList items = inputXml.SelectNodes("areas/area");

      int count = 0;
      string TeamProject = ConfigurationManager.AppSettings["TeamProject"];

      XmlNodeList areas = xmlInput.SelectNodes("//areas/area");
      foreach (XmlNode area in areas)
      {
        count++;
        Console.Write(".");
        if (area.Attributes["parent"] != null)
          TFSHelper.AddArea(tpc, TeamProject, area.InnerText, area.Attributes["parent"].Value);
        else
          TFSHelper.AddArea(tpc, TeamProject, area.InnerText);
      }
      Console.WriteLine(string.Format(" ({0} areas created)", count));
    }
    #endregion

    #region Iterations

    public static void RemoveIterations()
    {
      try
      {
        TfsTeamProjectCollection tpc = TfsConnect();
        string TeamProject = ConfigurationManager.AppSettings["TeamProject"];
        ICommonStructureService css = tpc.GetService<ICommonStructureService>();
        ProjectInfo project = css.GetProjectFromName(TeamProject);
        NodeInfo[] hierarchy = css.ListStructures(project.Uri);
        XmlElement tree = css.GetNodesXml(new string[] { hierarchy[1].Uri }, true);
        string parentUri = tree.FirstChild.Attributes["NodeID"].Value;

        // Enumerate nodes

        if (tree.HasChildNodes)
        {
          XmlNode childrenNode = tree.FirstChild;
          if (childrenNode.HasChildNodes && childrenNode.FirstChild.HasChildNodes)
          {
            string[] nodes = new string[childrenNode.ChildNodes[0].ChildNodes.Count];
            for (int i = 0; i < childrenNode.ChildNodes[0].ChildNodes.Count; i++)
            {
              XmlNode node = childrenNode.ChildNodes[0].ChildNodes[i];
              nodes[i] = node.Attributes["NodeID"].Value;
            }
            css.DeleteBranches(nodes, parentUri);
          }
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine("Error: " + ex.Message);
        throw;
      }
    }

    public static void CreateIterations()
    {
      try
      {
        TfsTeamProjectCollection tpc = TfsConnect();

        // Load iteration details

        DateTime startDate = DateTime.Today;
        int TotalReleases = int.Parse(ConfigurationManager.AppSettings["TotalReleases"]);
        int SprintsPerRelease = int.Parse(ConfigurationManager.AppSettings["SprintsPerRelease"]);
        int WeeksPerSprint = int.Parse(ConfigurationManager.AppSettings["WeeksPerSprint"]);
        bool ResetSprintsEachRelease = bool.Parse(ConfigurationManager.AppSettings["ResetSprintsEachRelease"]);

        int count = 0;
        string TeamProject = ConfigurationManager.AppSettings["TeamProject"];

        // Get service and hierarchy

        ICommonStructureService4 css = tpc.GetService<ICommonStructureService4>();
        ProjectInfo project = css.GetProjectFromName(TeamProject);
        NodeInfo[] hierarchy = css.ListStructures(project.Uri);
        XmlElement tree;
        tree = css.GetNodesXml(new string[] { hierarchy[1].Uri }, true);
        string parentUri = tree.FirstChild.Attributes["NodeID"].Value;
        int lastSprint = 1;

        for (int release = 1; release <= TotalReleases; release++)
        {
          string releaseUri = css.CreateNode("Release " + release.ToString(), parentUri);
          css.SetIterationDates(releaseUri, startDate, startDate.AddDays(SprintsPerRelease * WeeksPerSprint * 7 - 1));
          for (int sprint = 1; sprint <= SprintsPerRelease; sprint++)
          {
            string sprintUri;
            count++;
            Console.Write(".");
            if (ResetSprintsEachRelease)
              sprintUri = css.CreateNode("Sprint " + sprint.ToString(), releaseUri);
            else
              sprintUri = css.CreateNode("Sprint " + lastSprint, releaseUri);
            css.SetIterationDates(sprintUri, startDate, startDate.AddDays(WeeksPerSprint * 7 - 1));
            startDate = startDate.AddDays(WeeksPerSprint * 7);
            lastSprint++;
          }
        }
        Console.WriteLine(string.Format(" ({0} releases, {1} sprints created)", TotalReleases, count));
      }
      catch { }
    }

    public static void SetPlanningIterations()
    {
      TfsTeamProjectCollection tpc = TfsConnect();

      string TeamProject = ConfigurationManager.AppSettings["TeamProject"];
      var PlanningSprints = new List<string>(ConfigurationManager.AppSettings["PlanningIterations"].Split(new char[] { ';' }));

      // Get services

      ICommonStructureService4 css = tpc.GetService<ICommonStructureService4>();
      ProjectInfo project = css.GetProjectFromName(TeamProject);
      TeamSettingsConfigurationService teamConfigService = tpc.GetService<TeamSettingsConfigurationService>();
      var teamConfigs = teamConfigService.GetTeamConfigurationsForUser(new string[] { project.Uri });
      foreach (TeamConfiguration teamConfig in teamConfigs)
      {
        if (teamConfig.IsDefaultTeam)
        {
          TeamSettings settings = teamConfig.TeamSettings;
          settings.IterationPaths = PlanningSprints.ToArray();
          teamConfigService.SetTeamSettings(teamConfig.TeamId, settings);
        }
      }
    }
    #endregion

    #region Work Items
    public static void DestroyWorkItems(bool quiet)
    {
      TfsTeamProjectCollection tpc = TfsConnect();

      string TeamProject = ConfigurationManager.AppSettings["TeamProject"];

      // Get work items

      WorkItemStore store = new WorkItemStore(tpc);
      Project project = store.Projects[TeamProject];
      string wiql = "SELECT [System.Id], [System.Title] FROM WorkItems " + "WHERE [System.TeamProject] = '" + TeamProject + "'";
      WorkItemCollection collection = store.Query(wiql);

      // Get list of work items to destroy (everything but Sprint)

      ArrayList destroyIds = new ArrayList();
      for (int i = 0; i < collection.Count; i++)
      {
        WorkItem wi = collection[i];
        destroyIds.Add(wi.Id);
      }

      if (destroyIds.Count < 1)
      {
        return;
      }

      // Convert to int array

      int[] ids = new int[destroyIds.Count];
      for (int i = 0; i < destroyIds.Count; i++)
      {
        ids[i] = Convert.ToInt32(destroyIds[i]);
      }

      // Destroy work items

      IEnumerable<WorkItemOperationError> errors = store.DestroyWorkItems(ids);
      List<WorkItemOperationError> errorList = new List<WorkItemOperationError>(errors);
      if (errorList.Count > 0)
      {
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < errorList.Count; i++)
        {
          builder.AppendLine(string.Format("> Error destroying work item {0}, {1}", errorList[i].Id, errorList[i].Exception.Message));
        }
        Console.WriteLine(builder.ToString());
      }
    }

    public static void CreateWorkItems()
    {
      TfsTeamProjectCollection tpc = TfsConnect();

      int count = 0;
      Hashtable workItems = new Hashtable();
      string TeamProject = ConfigurationManager.AppSettings["TeamProject"];
      string ProductOwner = ConfigurationManager.AppSettings["ProductOwner"];

      // Validate that it's the right kind of team project

      WorkItemStore store = new WorkItemStore(tpc);
      Project project = store.Projects[TeamProject];
      if (!project.WorkItemTypes.Contains("Product Backlog Item"))
      {
        Console.WriteLine("This team project was not created using the Visual Studio Scrum process template.");
        return;
      }

      // Load work items

      XmlDocument xmlInput = new XmlDocument();
      xmlInput.Load(ConfigurationManager.AppSettings["InputFile"]);
      XmlNodeList items = xmlInput.SelectNodes("//items/item");
      foreach (XmlNode item in items)
      {
        Console.Write(".");
        string WIT = item["type"].InnerText.Trim().ToLower();
        string title = item["title"].InnerText;
        string state;
        if (item["state"] == null)
        {
          state = "approved";
        }
        else
        {
          state = item["state"].InnerText.ToLower();
        }
        WorkItem wi = new WorkItem(project.WorkItemTypes[WIT]);
        wi.Title = title;
        if (item["description"] != null)
        {
          wi.Description = item["description"].InnerText;
        }
        else
        {
          wi.Description = title;
        }
        if (item["reprosteps"] != null)
        {
          wi.Fields["Microsoft.VSTS.TCM.ReproSteps"].Value = item["reprosteps"].InnerText;
        }
        if (item["acceptancecriteria"] != null)
        {
          wi.Fields["Microsoft.VSTS.Common.AcceptanceCriteria"].Value = item["acceptancecriteria"].InnerText;
        }
        wi.AreaPath = TeamProject + item["area"].InnerText;
        if (item["iteration"] != null)
        {
          wi.IterationPath = TeamProject + item["iteration"].InnerText;
        }
        wi.Fields["System.AssignedTo"].Value = ProductOwner;
        if (state == "approved" || state == "active")
        {
          if (item["businessvalue"] != null && item["businessvalue"].InnerText != "")
          {
            wi.Fields["Microsoft.VSTS.Common.BusinessValue"].Value = item["businessvalue"].InnerText;
          }
          if (item["effort"] != null && item["effort"].InnerText != "")
          {
            wi.Fields["Microsoft.VSTS.Scheduling.Effort"].Value = item["effort"].InnerText;
          }
          if (item["order"] != null && item["order"].InnerText != "")
          {
            wi.Fields["Microsoft.VSTS.Common.BacklogPriority"].Value = item["order"].InnerText;
          }
        }

        ArrayList ValidationResult = wi.Validate();
        if (ValidationResult.Count > 0)
        {
          Console.WriteLine(ValidationResult[0].ToString());
        }

        wi.Save();
        count++;
        workItems.Add(wi.Id, wi.Title.Trim().ToLower());

        // linked items?

        if (item["parent"] != null)
        {
          string parent = item["parent"].InnerText.Trim().ToLower();
          int parentId = 0;
          foreach (DictionaryEntry candidate in workItems)
          {
            if (candidate.Value.ToString() == parent)
            {
              parentId = Convert.ToInt32(candidate.Key);
            }
          }
          if (parentId > 0)
          {
            // lookup child link type

            WorkItemLinkTypeEnd childLink = store.WorkItemLinkTypes.LinkTypeEnds["Parent"];
            wi.WorkItemLinks.Add(new WorkItemLink(childLink, parentId));
            wi.Save();
          }
        }

        if (state == "approved")
        {
          wi.State = "Approved";
          wi.Save();
        }
        else if (state == "active")
        {
          wi.State = "Active";
          wi.Save();
        }
        else if (state == "committed")
        {
          wi.State = "Committed";
          wi.Save();
        }
        else if (state == "done")
        {
          wi.State = "Done";
          wi.Save();
        }
      }

      // Done

      Console.WriteLine(string.Format(" ({0} work items created)", count));
    }
    #endregion

  }
}
